#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
# Cisco VPN Tunnel metrics plugin
#
# Author: Th.L.
# Date  : 2017-12-29
#

# key a       green      11/a       21/a       31/a       41/a       12/a       22/a       32/a       42/a
colors_a = ['#80F000', '#a500ff', '#ffc600', '#00ffb2', '#0075ff', '#cc00ff', '#ffd600', '#00ffff', '#0047ff',
            # 13/a       23/a       33/a       43/a       14/a       24/a       34/a       44/a       15/a
            '#f900ff', '#ffed00', '#00e8ff', '#000aff', '#ff4c00', '#e2ff00', '#00d1ff', '#4200ff', '#ff7a00',
            # 25/a       35/a       45/a       16/a       26/a       36/a       46/a       51/a       52/a
            '#bcff00', '#00b2ff', '#6000ff', '#ffa000', '#7fff00', '#0093ff', '#7f00ff', '#7f7f7f', '#7f4a26',
            # 53/a
            '#8c531c']
# key b       green      11/b       21/b       31/b       41/b       12/b       22/b       32/b       42/b
colors_b = ['#80F000', '#c966ff', '#cc9f00', '#00cc8e', '#66acff', '#e066ff', '#ccab00', '#00cccc', '#6690ff',
            # 13/b       23/b       33/b       43/b       14/b       24/b       34/b       44/b       15/b
            '#fb66ff', '#ccbd00', '#00b9cc', '#666cff', '#ff9366', '#b5cc00', '#00a7cc', '#8d66ff', '#ffaf66',
            # 25/b       35/b       45/b       16/b       26/b       36/b       46/b       51/b       52/b
            '#96cc00', '#008ecc', '#a066ff', '#ffc666', '#66cc00', '#0076cc', '#b266ff', '#7f7f7f', '#7f5f49',
            # 53/b
            '#8c6a48']


def cisco_vpn_tunnel_render_uptime(uptime):  # expects time in seconds
    m, s = divmod(uptime, 60)   # break in seconds / minutes
    h, m = divmod(m, 60)        # break in mintes / hours
    if h >= 24:                 # more then one day
        d, h = divmod(h, 24)    # break in hours / days
    else:
        return '%02d:%02d:%02d' % (h, m, s)
    if d >= 365:                # more the one year
        y, d = divmod(d, 365)   # break in days / years
        return '%dy %dd %02d:%02d:%02d' % (y, d, h, m, s)
    else:
        return '%dd %02d:%02d:%02d' % (d, h, m, s)

#####################################################################################################################
#
# define units for cisco_vpm_tunnel perfdata
#
#####################################################################################################################


unit_info['active_time'] = {
    'title': _('Last update'),
    'description': _('SA active time'),
    'symbol': _(''),
    'render': lambda v: cisco_vpn_tunnel_render_uptime(v),
    'stepping': 'time',  # for vertical graph labels
}


#####################################################################################################################
#
# define metrics for bgp peer perfdata
#
#####################################################################################################################


metric_info['cisco_vpn_tunnel_cikeTunActiveTime'] = {
    'title': _('IKE active time'),
    'help': _(''),
    #'unit': 'active_time',
    'unit': 's',
    'color': colors_a[0],
}
metric_info['cisco_vpn_tunnel_cikeTunInOctets'] = {
    'title': _('IKE Bytes in'),
    'unit': 'bytes/s',
    'color': colors_a[1],
}
metric_info['cisco_vpn_tunnel_cikeTunOutOctets'] = {
    'title': _('IKE Bytes out'),
    'help': _(''),
    'unit': 'bytes/s',
    'color': colors_a[2],
}
metric_info['cisco_vpn_tunnel_cikeTunInPkts'] = {
    'title': _('IKE packets in'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_a[3],
}
metric_info['cisco_vpn_tunnel_cikeTunOutPkts'] = {
    'title': _('IKE packets out'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_a[4],
}
metric_info['cisco_vpn_tunnel_cikeTunInDropPkts'] = {
    'title': _('IKE packets dropped in'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_a[5],
}
metric_info['cisco_vpn_tunnel_cikeTunOutDropPkts'] = {
    'title': _('IKE packets dropped out'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_a[6],
}

metric_info['cisco_vpn_tunnel_cikeTunInNotifys'] = {
    'title': _('IKE in notifies'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[7],
}
metric_info['cisco_vpn_tunnel_cikeTunOutNotifys'] = {
    'title': _('IKE out notifies'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[8],
}
metric_info['cisco_vpn_tunnel_cikeTunInP2Exchgs'] = {
    'title': _('IKE in phase 2 exchanges'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[9],
}
metric_info['cisco_vpn_tunnel_cikeTunOutP2Exchgs'] = {
    'title': _('IKE out phase 2 exchanges'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[10],
}
metric_info['cisco_vpn_tunnel_cikeTunInP2ExchgInvalids'] = {
    'title': _('IKE in phase 2 exchanges invalid'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[11],
}
metric_info['cisco_vpn_tunnel_cikeTunOutP2ExchgInvalids'] = {
    'title': _('IKE out phase 2 exchanges invalid'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[12],
}
metric_info['cisco_vpn_tunnel_cikeTunInP2ExchgRejects'] = {
    'title': _('IKE in phase 2 exchanges rejected'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[13],
}
metric_info['cisco_vpn_tunnel_cikeTunOutP2ExchgRejects'] = {
    'title': _('IKE out phase 2 exchanges rejected'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[14],
}
metric_info['cisco_vpn_tunnel_cikeTunInP2SaDelRequests'] = {
    'title': _('IKE in phase 2 SA delete requests'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[15],
}
metric_info['cisco_vpn_tunnel_cikeTunOutP2SaDelRequests'] = {
    'title': _('IKE out phase 2 SA delete requests'),
    'help': _(''),
    'unit': 'count',
    'color': colors_a[16],
}


# IPSec counter

metric_info['cisco_vpn_tunnel_cipSecTunActiveTime'] = {
    'title': _('IPSec active time'),
    'help': _(''),
    'unit': 's',
    'color': colors_b[0],
}
metric_info['cisco_vpn_tunnel_cipSecTunHcInOctets'] = {
    'title': _('IPSec Bytes in'),
    'help': _(''),
    'unit': 'bytes/s',
    'color': colors_b[1],
}
metric_info['cisco_vpn_tunnel_cipSecTunHcOutOctets'] = {
    'title': _('IPSec Bytes out'),
    'help': _(''),
    'unit': 'bytes/s',
    'color': colors_b[2],
}
metric_info['cisco_vpn_tunnel_cipSecTunInPkts'] = {
    'title': _('IPSec packets in'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[3],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutPkts'] = {
    'title': _('IPSec packets out'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[4],
}
metric_info['cisco_vpn_tunnel_cipSecTunInDropPkts'] = {
    'title': _('IPSec packets dropped in'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[5],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutDropPkts'] = {
    'title': _('IPSec packets dropped out'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[6],
}

metric_info['cisco_vpn_tunnel_cipSecTunHcInDecompOctets'] = {
    'title': _('IPSec in decompressed octets'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[7],
}
metric_info['cisco_vpn_tunnel_cipSecTunHcOutUncompOctets'] = {
    'title': _('IPSec out compressed octets'),
    'help': _(''),
    'unit': '1/s',
    'color': colors_b[8],
}
metric_info['cisco_vpn_tunnel_cipSecTunInAuths'] = {
    'title': _('IPSec in authentication\'s'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[9],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutAuths'] = {
    'title': _('IPSec out authentication\'s'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[10],
}
metric_info['cisco_vpn_tunnel_cipSecTunInAuthFails'] = {
    'title': _('IPSec in authentication\'s failed'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[11],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutAuthFails'] = {
    'title': _('IPSec out authentication\'s failed'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[12],
}
metric_info['cisco_vpn_tunnel_cipSecTunInDecrypts'] = {
    'title': _('IPSec in decryption\'s'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[13],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutEncrypts'] = {
    'title': _('IPSec out encryption\'s'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[14],
}
metric_info['cisco_vpn_tunnel_cipSecTunInDecryptFails'] = {
    'title': _('IPSec in decryption\'s failed'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[15],
}
metric_info['cisco_vpn_tunnel_cipSecTunOutEncryptFails'] = {
    'title': _('IPSec out encryption\'s failed'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[16],
}
metric_info['cisco_vpn_tunnel_cipSecTunInReplayDropPkts'] = {
    'title': _('IPSec in replay packets dropped'),
    'help': _(''),
    'unit': 'count',
    'color': colors_b[17],
}


######################################################################################################################
#
# map bgp peer perfdata to metric, not really necessary but makes sure to use the right metrics
#
######################################################################################################################


check_metrics['check_mk-cisco_vpn_tunnel'] = {
    'cikeTunInOctets': {'name': 'cisco_vpn_tunnel_cikeTunInOctets'},
    'cikeTunOutOctets': {'name': 'cisco_vpn_tunnel_cikeTunOutOctets'},
    'cikeTunInPkts': {'name': 'cisco_vpn_tunnel_cikeTunInPkts'},
    'cikeTunOutPkts': {'name': 'cisco_vpn_tunnel_cikeTunOutPkts'},
    'cikeTunInDropPkts': {'name': 'cisco_vpn_tunnel_cikeTunInDropPkts'},
    'cikeTunOutDropPkts': {'name': 'cisco_vpn_tunnel_cikeTunOutDropPkts'},
    'cikeTunInNotifys': {'name': 'cisco_vpn_tunnel_cikeTunInNotifys'},
    'cikeTunOutNotifys': {'name': 'cisco_vpn_tunnel_cikeTunOutNotifys'},
    'cikeTunInP2Exchgs': {'name': 'cisco_vpn_tunnel_cikeTunInP2Exchgs', 'auto_graph' : False},
    'cikeTunOutP2Exchgs': {'name': 'cisco_vpn_tunnel_cikeTunOutP2Exchgs'},
    'cikeTunInP2ExchgInvalids': {'name': 'cisco_vpn_tunnel_cikeTunInP2ExchgInvalids'},
    'cikeTunOutP2ExchgInvalids': {'name': 'cisco_vpn_tunnel_cikeTunOutP2ExchgInvalids'},
    'cikeTunInP2ExchgRejects': {'name': 'cisco_vpn_tunnel_cikeTunInP2ExchgRejects'},
    'cikeTunOutP2ExchgRejects': {'name': 'cisco_vpn_tunnel_cikeTunOutP2ExchgRejects'},
    'cikeTunInP2SaDelRequests': {'name': 'cisco_vpn_tunnel_cikeTunInP2SaDelRequests'},
    'cikeTunOutP2SaDelRequests': {'name': 'cisco_vpn_tunnel_cikeTunOutP2SaDelRequests'},
    'cikeTunActiveTime': {'name': 'cisco_vpn_tunnel_cikeTunActiveTime'},

    'cipSecTunHcInOctets': {'name': 'cisco_vpn_tunnel_cipSecTunHcInOctets'},
    'cipSecTunHcOutOctets': {'name': 'cisco_vpn_tunnel_cipSecTunHcOutOctets'},
    'cipSecTunHcInDecompOctets': {'name': 'cisco_vpn_tunnel_cipSecTunHcInDecompOctets'},
    'cipSecTunHcOutUncompOctets': {'name': 'cisco_vpn_tunnel_cipSecTunHcOutUncompOctets'},
    'cipSecTunInPkts': {'name': 'cisco_vpn_tunnel_cipSecTunInPkts'},
    'cipSecTunOutPkts': {'name': 'cisco_vpn_tunnel_cipSecTunOutPkts'},
    'cipSecTunInDropPkts': {'name': 'cisco_vpn_tunnel_cipSecTunInDropPkts'},
    'cipSecTunOutDropPkts': {'name': 'cisco_vpn_tunnel_cipSecTunOutDropPkts'},
    'cipSecTunInAuths': {'name': 'cisco_vpn_tunnel_cipSecTunInAuths'},
    'cipSecTunOutAuths': {'name': 'cisco_vpn_tunnel_cipSecTunOutAuths'},
    'cipSecTunInAuthFails': {'name': 'cisco_vpn_tunnel_cipSecTunInAuthFails'},
    'cipSecTunOutAuthFails': {'name': 'cisco_vpn_tunnel_cipSecTunOutAuthFails'},
    'cipSecTunInDecrypts': {'name': 'cisco_vpn_tunnel_cipSecTunInDecrypts'},
    'cipSecTunOutEncrypts': {'name': 'cisco_vpn_tunnel_cipSecTunOutEncrypts'},
    'cipSecTunInDecryptFails': {'name': 'cisco_vpn_tunnel_cipSecTunInDecryptFails'},
    'cipSecTunOutEncryptFails': {'name': 'cisco_vpn_tunnel_cipSecTunOutEncryptFails'},
    'cipSecTunInReplayDropPkts': {'name': 'cisco_vpn_tunnel_cipSecTunInReplayDropPkts'},
    'cipSecTunActiveTime': {'name': 'cisco_vpn_tunnel_cipSecTunActiveTime'},

}

######################################################################################################################
#
# how to graph perdata for cisco_vpn_tunnel
#
######################################################################################################################


graph_info.append({
    'title': _('IKE active time'),
    'metrics': [
        ('cisco_vpn_tunnel_cikeTunActiveTime', 'area'),
    ],
})
graph_info.append({
    'title': _('IKE Bytes/s'),
    'metrics': [
        ('cisco_vpn_tunnel_cikeTunOutOctets', '-area'),
        ('cisco_vpn_tunnel_cikeTunInOctets', 'area'),
    ],
})
graph_info.append({
    'title': _('IKE packets/s'),
    'metrics': [
        ('cisco_vpn_tunnel_cikeTunOutDropPkts', '-line'),
        ('cisco_vpn_tunnel_cikeTunInDropPkts', 'line'),
        ('cisco_vpn_tunnel_cikeTunOutPkts', '-line'),
        ('cisco_vpn_tunnel_cikeTunInPkts', 'line'),
    ],
})

# graph_info.append({
#     'title': _('IKE in data'),
#     'metrics': [
#         ('cisco_vpn_tunnel_cikeTunInNotifys', 'line'),
# #        ('cisco_vpn_tunnel_cikeTunInP2Exchgs', 'line'),
#         ('cisco_vpn_tunnel_cikeTunInP2ExchgInvalids', 'line'),
#         ('cisco_vpn_tunnel_cikeTunInP2ExchgRejects', 'line'),
#         ('cisco_vpn_tunnel_cikeTunInP2SaDelRequests', 'line'),
#     ],
# })
#
# graph_info.append({
#     'title': _('IKE out data'),
#     'metrics': [
#
#         ('cisco_vpn_tunnel_cikeTunOutNotifys', '-line'),
# #        ('cisco_vpn_tunnel_cikeTunOutP2Exchgs', '-line'),
#         ('cisco_vpn_tunnel_cikeTunOutP2ExchgInvalids', '-line'),
#         ('cisco_vpn_tunnel_cikeTunOutP2ExchgRejects', '-line'),
#         ('cisco_vpn_tunnel_cikeTunOutP2SaDelRequests', '-line'),
#     ],
# })

graph_info.append({
    'title': _('IPSec active time'),
    'metrics': [
        ('cisco_vpn_tunnel_cipSecTunActiveTime', 'area'),
    ],
})
graph_info.append({
    'title': _('IPSec Bytes/s'),
    'metrics': [
        ('cisco_vpn_tunnel_cipSecTunHcOutOctets', '-area'),
        ('cisco_vpn_tunnel_cipSecTunHcInOctets', 'area'),
    ],
})
graph_info.append({
    'title': _('IPSec packets/s'),
    'metrics': [
        ('cisco_vpn_tunnel_cipSecTunOutDropPkts', '-stack'),
        ('cisco_vpn_tunnel_cipSecTunInDropPkts', 'stack'),
        ('cisco_vpn_tunnel_cipSecTunOutPkts', '-stack'),
        ('cisco_vpn_tunnel_cipSecTunInPkts', 'stack'),
    ],
})

# graph_info.append({
#     'title': _('IPSec in data'),
#     'metrics': [
# #        ('cisco_vpn_tunnel_cipSecTunHcInDecompOctets', 'line'),
# #        ('cisco_vpn_tunnel_cipSecTunInAuths', 'line'),
#         ('cisco_vpn_tunnel_cipSecTunInAuthFails', 'line'),
# #        ('cisco_vpn_tunnel_cipSecTunInDecrypts', 'line'),
#         ('cisco_vpn_tunnel_cipSecTunInDecryptFails', 'line'),
#         ('cisco_vpn_tunnel_cipSecTunInReplayDropPkts', 'line'),
#     ],
# })
#
# graph_info.append({
#     'title': _('IPSec out data'),
#     'metrics': [
# #        ('cisco_vpn_tunnel_cipSecTunHcOutUncompOctets', '-line'),
# #        ('cisco_vpn_tunnel_cipSecTunOutAuths', '-line'),
#         ('cisco_vpn_tunnel_cipSecTunOutAuthFails', '-line'),
# #        ('cisco_vpn_tunnel_cipSecTunOutEncrypts', '-line'),
#         ('cisco_vpn_tunnel_cipSecTunOutEncryptFails', '-line'),
#     ],
# })


######################################################################################################################
#
# define perf-o-meter for cisco_vpn_tunnel uptime active time
#
######################################################################################################################

perfometer_info.append(('stacked', [
    {
        'type': 'logarithmic',
        'metric': 'cisco_vpn_tunnel_cikeTunActiveTime',
        'half_value': 2592000.0,
        'exponent': 2,
    },
    {
        'type': 'logarithmic',
        'metric': 'cisco_vpn_tunnel_cipSecTunActiveTime',
        'half_value': 2592000.0,
        'exponent': 2,
    },
]))
